<?php namespace RVR\Repository;

use Container;
use DateTime;
use Generator;
use RVR\PersistentData\Model\Community;
use RVR\PersistentData\Model\Event;
use RVR\PersistentData\Model\User;
use SokoWeb\Database\Query\Select;

class EventRepository
{
    public function getById(int $id): ?Event
    {
        return Container::$persistentDataManager->selectFromDbById($id, Event::class);
    }

    public function getBySlug(string $slug): ?Event
    {
        return \Container::$persistentDataManager->selectFromDbBySlug($slug, Event::class);
    }

    public function getAllByCommunity(Community $community, bool $useRelations = false, array $withRelations = []): Generator
    {
        $select = $this->selectAllByCommunity($community);

        yield from Container::$persistentDataManager->selectMultipleFromDb($select, Event::class, $useRelations, $withRelations);
    }

    public function countAllByCommunity(Community $community): int
    {
        return $this->selectAllByCommunity($community)->count();
    }

    public function getUpcomingByCommunity(Community $community, DateTime $from, int $limit, bool $useRelations = false, array $withRelations = []): Generator
    {
        $select = $this->selectAllByCommunity($community);
        $select->where('end', '>', $from->format('Y-m-d H:i:s'));
        $select->orderBy('start', 'ASC');
        $select->limit($limit, 0);

        yield from Container::$persistentDataManager->selectMultipleFromDb($select, Event::class, $useRelations, $withRelations);
    }

    public function getUpcomingByUser(User $user, DateTime $from, int $limit, bool $useRelations = false, array $withRelations = []): Generator
    {
        $select = $this->selectAllByUser($user);
        $select->where('end', '>', $from->format('Y-m-d H:i:s'));
        $select->orderBy('start', 'ASC');
        $select->limit($limit, 0);

        yield from Container::$persistentDataManager->selectMultipleFromDb($select, Event::class, $useRelations, $withRelations);
    }

    public function getPagedByCommunity(Community $community, int $page, int $itemsPerPage, bool $useRelations = false, array $withRelations = []): Generator
    {
        $select = $this->selectAllByCommunity($community);
        $select->orderBy('start', 'DESC');
        $select->paginate($page, $itemsPerPage);

        yield from Container::$persistentDataManager->selectMultipleFromDb($select, Event::class, $useRelations, $withRelations);
    }

    public function searchByTitle(Community $community, string $title): Generator
    {
        $select = $this->selectAllByCommunity($community);
        $select->where('title', 'LIKE', '%' . $title . '%');
        $select->orderBy('start', 'DESC');
        $select->limit(10);

        yield from Container::$persistentDataManager->selectMultipleFromDb($select, Event::class);
    }

    private function selectAllByCommunity(Community $community)
    {
        $select = new Select(Container::$dbConnection, Event::getTable());
        $select->where('community_id', '=', $community->getId());
        return $select;
    }

    private function selectAllByUser(User $user)
    {
        $select = new Select(Container::$dbConnection, Event::getTable());
        $select->innerJoin('communities', ['communities', 'id'], '=', ['events', 'community_id']);
        $select->innerJoin('community_members', ['communities', 'id'], '=', ['community_members', 'community_id']);
        $select->where(['community_members', 'user_id'], '=', $user->getId());
        return $select;
    }
}
